@@ -26,9 +26,6 @@
 #include <string.h>
 
 #ifdef G_OS_WIN32
-#ifdef DATADIR
-#undef DATADIR
-#endif
 #include <io.h>
 #include <conio.h>
 #if !(_WIN32_WINNT >= 0x0500)
@@ -38,12 +35,17 @@
 #endif
 
 #define THUMBNAIL_SIZE 128
+#define DEFAULT_SLEEP_TIME (15 * G_USEC_PER_SEC) /* 15 seconds */
+
+static gboolean finished = TRUE;
 
 static gint size = THUMBNAIL_SIZE;
+static gboolean time_limit = TRUE;
 static const gchar **file_arguments;
 
 static const GOptionEntry goption_options[] = {
 	{ "size", 's', 0, G_OPTION_ARG_INT, &size, NULL, "SIZE" },
+        { "no-limit", 'l', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &time_limit, "Don't limit the thumbnailing time to 15 seconds", NULL },
 	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &file_arguments, NULL, "<input> <ouput>" },
 	{ NULL }
 };
@@ -55,6 +57,41 @@ struct AsyncData {
 	gboolean     success;
 };
 
+/* Time monitor: copied from totem */
+G_GNUC_NORETURN static gpointer
+time_monitor (gpointer data)
+{
+        const gchar *app_name;
+
+        g_usleep (DEFAULT_SLEEP_TIME);
+
+        if (finished)
+                g_thread_exit (NULL);
+
+        app_name = g_get_application_name ();
+        if (app_name == NULL)
+                app_name = g_get_prgname ();
+        g_print ("%s couldn't process file: '%s'\n"
+                 "Reason: Took too much time to process.\n",
+                 app_name,
+                 (const char *) data);
+
+        exit (0);
+}
+
+static void
+time_monitor_start (const char *input)
+{
+        finished = FALSE;
+        g_thread_create (time_monitor, (gpointer) input, FALSE, NULL);
+}
+
+static void
+time_monitor_stop (void)
+{
+        finished = TRUE;
+}
+
 static void
 delete_temp_file (GFile *file)
 {
@@ -140,8 +177,7 @@ evince_thumbnail_pngenc_get (EvDocument *document, const char *thumbnail, int si
 	ev_document_get_page_size (document, 0, &width, &height);
 
 	rc = ev_render_context_new (page, 0, size / width);
-	pixbuf = ev_document_thumbnails_get_thumbnail (EV_DOCUMENT_THUMBNAILS (document),
-						       rc, FALSE);
+	pixbuf = ev_document_get_thumbnail (document, rc);
 	g_object_unref (rc);
 	g_object_unref (page);
 	
@@ -156,7 +192,7 @@ evince_thumbnail_pngenc_get (EvDocument *document, const char *thumbnail, int si
 			gchar *overlaid_icon_path = g_build_filename (dir, "share", "evince", overlaid_icon_name, NULL);
 			g_free (dir);
 #else
-			gchar *overlaid_icon_path = g_strdup_printf ("%s/%s", DATADIR, overlaid_icon_name);
+			gchar *overlaid_icon_path = g_strdup_printf ("%s/%s", EVINCEDATADIR, overlaid_icon_name);
 #endif
 			overlaid_pixbuf = gdk_pixbuf_new_from_file (overlaid_icon_path, NULL);
 			g_free (overlaid_icon_path);
@@ -273,11 +309,8 @@ main (int argc, char *argv[])
 		return -2;
 	}
 
-	if (!EV_IS_DOCUMENT_THUMBNAILS (document)) {
-		g_object_unref (document);
-		ev_shutdown ();
-		return -2;
-	}
+        if (time_limit)
+                time_monitor_start (input);
 
 	if (EV_IS_ASYNC_RENDERER (document)) {
 		struct AsyncData data;
@@ -305,6 +338,7 @@ main (int argc, char *argv[])
 		return -2;
 	}
 
+        time_monitor_stop ();
 	g_object_unref (document);
         ev_shutdown ();
 
